/*
 * JZ4730 CPM server.
 *
 * Copyright (C) 2018, 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/cpm-jz4730.h>
#include <l4/devices/memory.h>

#include <l4/re/env.h>

#include <ipc/server.h>
#include "cpm_server.h"

/* Virtual addresses for the CPM and LCD register blocks. */

static l4_addr_t cpm_virt_base = 0, cpm_virt_base_end = 0;



/* Access to peripheral memory. */

static int setup_memory()
{
  if (get_memory("jz4730-cpm", &cpm_virt_base, &cpm_virt_base_end))
    return 1;

  return 0;
}



/* CPM server. */

class server_CPM : public CPM
{
  Cpm_jz4730_chip *_chip;

public:
  explicit server_CPM(Cpm_jz4730_chip *chip)
  : _chip(chip)
  {
  }

  long set_frequency(enum Clock_frequency_identifiers clock, uint32_t frequency)
  {
    _chip->set_frequency(clock, frequency);
    return L4_EOK;
  }

  long have_clock(enum Clock_identifiers clock, int *enabled)
  {
    *enabled = _chip->have_clock(clock);
    return L4_EOK;
  }

  long start_clock(enum Clock_identifiers clock)
  {
    _chip->start_clock(clock);
    return L4_EOK;
  }

  long stop_clock(enum Clock_identifiers clock)
  {
    _chip->stop_clock(clock);
    return L4_EOK;
  }

  long update_output_frequency()
  {
    _chip->update_output_frequency();
    return L4_EOK;
  }
};



/* Main program. */

int main(void)
{
  if (setup_memory()) return 1;

  /* Initialise the CPM abstraction. */

  Cpm_jz4730_chip cpm_device(cpm_virt_base, 3686400);

  /* Initialise and register a server object. */

  server_CPM obj(&cpm_device);
  l4_cap_idx_t server;

  if (ipc_server_bind("cpm", (l4_umword_t) &obj, &server)) return 1;

  /* Enter the IPC server loop. */

  ipc_server_loop(CPM_expected_items, &obj,
                  (ipc_server_handler_type) handle_CPM);
  return 0;
}
